@@ -1,31 +1,27 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-2000, Patrick Powell, San Diego, CA
+ * Copyright 1988-2002, Patrick Powell, San Diego, CA
  *     papowell@lprng.com
  * See LICENSE for conditions of use.
  *
  ***************************************************************************/
 
  static char *const _id =
-"$Id: lpd.c,v 5.23 2000/12/25 01:51:08 papowell Exp papowell $";
+"$Id: lpd.c,v 1.27 2002/04/01 17:54:52 papowell Exp $";
 
 
 #include "lp.h"
 #include "child.h"
 #include "fileopen.h"
-#include "gethostinfo.h"
-#include "getopt.h"
-#include "getprinter.h"
-#include "getqueue.h"
+#include "errorcodes.h"
 #include "initialize.h"
 #include "linksupport.h"
-#include "patchlevel.h"
-#include "permission.h"
+#include "lpd_logger.h"
+#include "getqueue.h"
+#include "getopt.h"
 #include "proctitle.h"
-#include "errorcodes.h"
-#include "krb5_auth.h"
-#include "lpd_secure.h"
+#include "lockfile.h"
 
 /* force local definitions */
 #undef EXTERN
@@ -37,13 +33,10 @@
 #define DEFS
 
 #include "lpd.h"
-#include "lpd_remove.h"
-#include "lpd_logger.h"
-#include "lpd_jobs.h"
-#include "lpd_rcvjob.h"
-#include "lpd_control.h"
-#include "lpd_status.h"
 
+ char* Lpd_listen_port_arg;	/* command line listent port value */
+ char* Lpd_port_arg;	/* command line port value */
+ char* Lpd_socket_arg; /* command line unix socket value */
 
 /**** ENDINCLUDE ****/
 
@@ -70,6 +63,7 @@ int main(int argc, char *argv[], char *envp[])
 	struct timeval timeval, *timeout;
 	int max_socks;		/* maximum number of sockets */
 	int n, m;	/* ACME?  Hmmm... well, ok */
+	int lockfd;	/* the lock file descriptor */
 	int err, newsock;
  	time_t last_time;	/* time that last Start_all was done */
  	time_t this_time;	/* current time */
@@ -78,10 +72,12 @@ int main(int argc, char *argv[], char *envp[])
 	int start_fd = 0;
 	int status_pid = 0;
 	int request_pipe[2], status_pipe[2];
-	int fork_failed;
+	int last_fork_pid_value;
 	struct line_list args;
+	struct sockaddr sinaddr;
 	char *s;
 	int first_scan = 1;
+	int unix_sock = 0;
 
 	Init_line_list( &args );
 	Is_server = 1;	/* we are the LPD server */
@@ -152,18 +148,12 @@ int main(int argc, char *argv[], char *envp[])
 	Initialize(argc, argv, envp, 'D' );
 	DEBUG1("Get_parms: UID_root %d, OriginalRUID %d", UID_root, OriginalRUID);
 
-	if( UID_root && OriginalRUID ){
+	if( UID_root && (OriginalRUID != ROOTUID) ){
 		FATAL(LOG_ERR) "lpd installed SETUID root and started by user %d! Possible hacker attack", OriginalRUID);
 	}
 
 	Setup_configuration();
 
-	if( Worker_LPD ){
-		Lpd_worker(argv,argc,Optind);
-		cleanup(0);
-	}
-
-
 	/* get the maximum number of servers allowed */
 	max_servers = Get_max_servers();
 	n = Get_max_fd();
@@ -179,31 +169,49 @@ int main(int argc, char *argv[], char *envp[])
 		Errorcode = JABORT;
 		LOGERR_DIE(LOG_ERR) "cannot chdir to /");
 	}
-
-	/*
-	 * This is the one of the two places where we need to be
-	 * root in order to open a socket
-	 */
-
-	sock = Link_listen();
-	DEBUG1("lpd: listening socket fd %d",sock);
-	if( sock < 0 ){
-		/*
-		 * try reading the lockfile
-		 */
-  		pid = Get_lpd_pid();
- 		if( pid > 0 && kill(pid,0) ) pid = 0;
-		Errorcode = 1;
-  		if( pid > 0 ){
-  			DIEMSG( _("Another print spooler is using TCP printer port, possibly lpd process '%d'"),
+	pid = Get_lpd_pid();
+#if defined(__CYGWIN__)
+	if( (pid > 0 && ( kill(pid,0) || (errno != ESRCH) )) {
+		DIEMSG( _("Another print spooler active, possibly lpd process '%d'"),
   				pid );
-  		} else {
- 			if( !UID_root ){
- 				DIEMSG("Not running with ROOT perms and trying to open port '%s'", Lpd_port_DYN );
- 			}
- 			DIEMSG( _("Another print spooler is using TCP printer port - not LPRng") );
-  		}
-		Errorcode = JSUCC;
+	}
+	lockfd = Lock_lpd_pid();
+	if( lockfd < 0 ){
+		DIEMSG( _("cannot open or lock lockfile - %s"), Errormsg(errno) );
+	}
+	Set_lpd_pid( lockfd );
+	close( lockfd );
+	lockfd = -1;
+#else
+	lockfd = Lock_lpd_pid();
+	if( lockfd < 0 ){
+		DIEMSG( _("Another print spooler active, possibly lpd process '%d'"),
+  				pid );
+	}
+	Set_lpd_pid( lockfd );
+#endif
+
+	s = Lpd_listen_port_arg;
+	if( ISNULL(s) ) s = Lpd_listen_port_DYN;
+	if( ISNULL(s) ) s = Lpd_port_DYN;
+	if( !ISNULL(s) && safestrcasecmp( s,"off") && strtol(s,0,0) ){
+		sock = Link_listen(s);
+		DEBUG1("lpd: listening socket fd %d",sock);
+		if( sock < 0 ){
+			Errorcode = 1;
+			DIEMSG("Cannot bind to lpd port '%s'", s);
+		}
+	}
+
+	s = Lpd_socket_arg;
+	if( ISNULL(s) ) s = Unix_socket_path_DYN;
+	if( !ISNULL(s) && safestrcasecmp( s,"off") ){
+		unix_sock = Unix_link_listen(s);
+		DEBUG1("lpd: unix listening socket fd %d, path '%s'",unix_sock, s);
+		if( unix_sock < 0 ){
+			Errorcode = 1;
+			DIEMSG("Cannot bind to UNIX socket '%s'", s );
+		}
 	}
 
 	/* setting nonblocking on the listening fd
@@ -252,7 +260,18 @@ int main(int argc, char *argv[], char *envp[])
 	 * Write the PID into the lockfile
 	 */
 
-	Set_lpd_pid();
+#if defined(__CYGWIN__)
+	lockfd = Lock_lpd_pid();
+	if( lockfd < 0 ) {
+	   DIEMSG( "Can't open lockfile for writing" );
+	}
+	Set_lpd_pid( lockfd );
+	close( lockfd );
+	lockfd = -1;
+#else
+	Set_lpd_pid( lockfd );
+#endif
+
 
 	if( Drop_root_DYN ){
 		Full_daemon_perms();
@@ -276,6 +295,10 @@ int main(int argc, char *argv[], char *envp[])
 		Max_open(status_pipe[0]); Max_open(status_pipe[1]);
 		Logger_fd = status_pipe[1];
 		DEBUG2( "lpd: fd status_pipe(%d,%d)",status_pipe[0],status_pipe[1]);
+		status_pid = Start_logger( status_pipe[0] );
+		if( status_pid < 0 ){
+			LOGERR_DIE(LOG_ERR) _("lpd: cannot start initial logger process") );
+		}
 	}
 
 	/* open a connection to logger */
@@ -293,7 +316,8 @@ int main(int argc, char *argv[], char *envp[])
 	/* set up the wait activity */
 
 	FD_ZERO( &defreadfds );
-	FD_SET( sock, &defreadfds );
+	if( sock > 0 ) FD_SET( sock, &defreadfds );
+	if( unix_sock > 0 ) FD_SET( unix_sock, &defreadfds );
 	FD_SET( request_pipe[0], &defreadfds );
 
 	/*
@@ -301,8 +325,8 @@ int main(int argc, char *argv[], char *envp[])
 	 */
 
 	last_time = time( (void *)0 );
-	fork_failed = start_fd = Start_all(first_scan);
-	Fork_error( fork_failed );
+	last_fork_pid_value = start_fd = Start_all(first_scan);
+	Fork_error( last_fork_pid_value );
 	if( start_fd > 0 ){
 		first_scan = 0;
 	}
@@ -326,15 +350,15 @@ int main(int argc, char *argv[], char *envp[])
 			if( pid == status_pid ){
 				status_pid = -1;
 			}
-			fork_failed = 1;
+			last_fork_pid_value = 1;
 		}
-		if( fork_failed > 0 && Logger_fd > 0 && status_pid < 0 ){
+		if( last_fork_pid_value > 0 && Logger_fd > 0 && status_pid < 0 ){
 			DEBUG1( "lpd: restarting logger process");
-			fork_failed = status_pid = Start_logger( status_pipe[0] );
-			Fork_error( fork_failed );
+			last_fork_pid_value = status_pid = Start_logger( status_pipe[0] );
+			Fork_error( last_fork_pid_value );
 			DEBUG1("lpd: status_pid %d", status_pid );
 		}
-		if( fork_failed < 0 ){
+		if( last_fork_pid_value < 0 ){
 			/* wait for 10 seconds then go in a loop */
 			memset(&timeval, 0, sizeof(timeval));
 			timeval.tv_sec = 10;
@@ -350,8 +374,8 @@ int main(int argc, char *argv[], char *envp[])
 				m, (int)timeval.tv_sec );
 			if( m >= Poll_time_DYN ){
 				if( Started_server || Force_poll_DYN ){
-					fork_failed = start_fd = Start_all(first_scan);
-					Fork_error( fork_failed );
+					last_fork_pid_value = start_fd = Start_all(first_scan);
+					Fork_error( last_fork_pid_value );
 					DEBUG1( "lpd: restarting poll, start_fd %d", start_fd);
 					if( start_fd > 0 ){
 						if( first_scan ) first_scan = 0;
@@ -370,13 +394,12 @@ int main(int argc, char *argv[], char *envp[])
 		DEBUG1("lpd: max_servers %d, active %d", max_servers, n );
 
 		/* allow a little space for people to send commands */
-		while( fork_failed > 0 && Servers_line_list.count > 0 && n < max_servers-4 ){ 
+		while( last_fork_pid_value > 0 && Servers_line_list.count > 0 && n < max_servers-4 ){ 
 			s = Servers_line_list.list[0];
 			DEBUG1("lpd: starting server '%s'", s );
 			Set_str_value(&args,PRINTER,s);
-			Set_str_value(&args,CALL,QUEUE);
-			fork_failed = pid = Start_worker( &args, 0 );
-			Fork_error( fork_failed );
+			last_fork_pid_value = pid = Start_worker( "queue",&args, 0 );
+			Fork_error( last_fork_pid_value );
 			Free_line_list(&args);
 			if( pid > 0 ){
 				Remove_line_list( &Servers_line_list, 0 );
@@ -385,13 +408,14 @@ int main(int argc, char *argv[], char *envp[])
 			}
 		}
 
-		DEBUG1("lpd: fork_failed %d, processes %d active, max %d",
-			fork_failed, n, max_servers );
+		DEBUG1("lpd: last_fork_pid_value %d, processes %d active, max %d",
+			last_fork_pid_value, n, max_servers );
 		/* do not accept incoming call if no worker available */
 		readfds = defreadfds;
-		if( n >= max_servers || fork_failed < 0 ){
+		if( n >= max_servers || last_fork_pid_value < 0 ){
 			DEBUG1( "lpd: not accepting requests" );
-			FD_CLR( sock, &readfds );
+			if( sock > 0 ) FD_CLR( sock, &readfds );
+			if( unix_sock > 0 ) FD_CLR( unix_sock, &readfds );
 		}
 
 		max_socks = sock+1;
@@ -453,21 +477,19 @@ int main(int argc, char *argv[], char *envp[])
 			}
 			continue;
 		} else if( m == 0 ){
-			DEBUG1( "lpd: signal or time out, fork_failed %d", fork_failed );
+			DEBUG1( "lpd: signal or time out, last_fork_pid_value %d", last_fork_pid_value );
 			/* we try to fork now */
-			if( fork_failed < 0 ) fork_failed = 1;
+			if( last_fork_pid_value < 0 ) last_fork_pid_value = 1;
 			continue;
 		}
-		if( FD_ISSET( sock, &readfds ) ){
-			struct sockaddr sinaddr;
+		if( sock > 0 && FD_ISSET( sock, &readfds ) ){
 			int len;
 			len = sizeof( sinaddr );
 			newsock = accept( sock, &sinaddr, &len );
 			err = errno;
 			DEBUG1("lpd: connection fd %d", newsock );
 			if( newsock > 0 ){
-				Set_str_value(&args,CALL,SERVER);
-				pid = Start_worker( &args, newsock );
+				pid = Start_worker( "server", &args, newsock );
 				if( pid < 0 ){
 					LOGERR(LOG_INFO) _("lpd: fork() failed") );
 					Write_fd_str( newsock, "\002Server load too high\n");
@@ -478,7 +500,28 @@ int main(int argc, char *argv[], char *envp[])
 				Free_line_list(&args);
 			} else {
 				errno = err;
-				LOGERR(LOG_INFO) _("Service_connection: accept on listening socket failed") );
+				LOGERR(LOG_INFO) _("lpd: accept on listening socket failed") );
+			}
+		}
+		if( unix_sock > 0 && FD_ISSET( unix_sock, &readfds ) ){
+			int len;
+			len = sizeof( sinaddr );
+			newsock = accept( unix_sock, &sinaddr, &len );
+			err = errno;
+			DEBUG1("lpd: unix socket connection fd %d", newsock );
+			if( newsock > 0 ){
+				pid = Start_worker( "server", &args, newsock );
+				if( pid < 0 ){
+					LOGERR(LOG_INFO) _("lpd: fork() failed") );
+					Write_fd_str( newsock, "\002Server load too high\n");
+				} else {
+					DEBUG1( "lpd: listener pid %d running", pid );
+				}
+				close( newsock );
+				Free_line_list(&args);
+			} else {
+				errno = err;
+				LOGERR(LOG_INFO) _("lpd: accept on listening socket failed") );
 			}
 		}
 		if( FD_ISSET( request_pipe[0], &readfds ) 
@@ -534,147 +577,6 @@ void Setup_log(char *logfile )
 	}
 }
 
-/***************************************************************************
- * Service_connection( struct line_list *args )
- *  Service the connection on the talk socket
- * 1. fork a connection
- * 2. Mother:  close talk and return
- * 2  Child:  close listen
- * 2  Child:  read input line and decide what to do
- *
- ***************************************************************************/
-
-void Service_connection( struct line_list *args )
-{
-	char input[SMALLBUFFER];
-	char buffer[LINEBUFFER];	/* for messages */
-	int len, talk;
-	int status;		/* status of operation */
-	int permission;
-	int port = 0;
-	struct sockaddr sinaddr;
-
-	Name = "SERVER";
-	setproctitle( "lpd %s", Name );
-	(void) plp_signal (SIGHUP, cleanup );
-
-	if( !(talk = Find_flag_value(args,INPUT,Value_sep)) ){
-		Errorcode = JABORT;
-		FATAL(LOG_ERR)"Service_connection: no talk fd"); 
-	}
-
-	DEBUG1("Service_connection: listening fd %d", talk );
-
-	Free_line_list(args);
-
-	/* make sure you use blocking IO */
-	Set_block_io(talk);
-
-	len = sizeof( sinaddr );
-	if( getpeername( talk, &sinaddr, &len ) ){
-		LOGERR_DIE(LOG_DEBUG) _("Service_connection: getpeername failed") );
-	}
-
-	if( sinaddr.sa_family == AF_INET ){
-		port = ((struct sockaddr_in *)&sinaddr)->sin_port;
-#if defined(IPV6)
-	} else if( sinaddr.sa_family == AF_INET6 ){
-		port = ((struct sockaddr_in6 * )&sinaddr)->sin6_port;
-#endif
-	} else {
-		FATAL(LOG_INFO) _("Service_connection: bad protocol family '%d'"), sinaddr.sa_family );
-	}
-
-	DEBUG2("Service_connection: socket %d, ip '%s' port %d", talk,
-		inet_ntop_sockaddr( &sinaddr, buffer, sizeof(buffer) ), ntohs( port ) );
-
-	/* get the remote name and set up the various checks */
-	Perm_check.addr = &sinaddr;
-
-	Get_remote_hostbyaddr( &RemoteHost_IP, &sinaddr );
-	Perm_check.remotehost  =  &RemoteHost_IP;
-	Perm_check.host = &RemoteHost_IP;
-	Perm_check.port =  ntohs(port);
-
-	len = sizeof( input ) - 1;
-	memset(input,0,sizeof(input));
-	DEBUG1( "Service_connection: starting read on fd %d", talk );
-
-	status = Link_line_read(ShortRemote_FQDN,&talk,
-		Send_job_rw_timeout_DYN,input,&len);
-	if( len >= 0 ) input[len] = 0;
-	DEBUG1( "Service_connection: read status %d, len %d, '%s'",
-		status, len, input );
-	if( len == 0 ){
-		DEBUG3( "Service_connection: zero length read" );
-		cleanup(0);
-	}
-	if( status ){
-		LOGERR_DIE(LOG_DEBUG) _("Service_connection: cannot read request") );
-	}
-	if( len < 2 ){
-		FATAL(LOG_INFO) _("Service_connection: bad request line '%s'"), input );
-	}
-
-	/* read the permissions information */
-
-	if( Perm_filters_line_list.count ){
-		Free_line_list(&Perm_line_list);
-		Merge_line_list(&Perm_line_list,&RawPerm_line_list,0,0,0);
-		Filterprintcap( &Perm_line_list, &Perm_filters_line_list, "");
-	}
-   
-	Perm_check.service = 'X';
-
-	permission = Perms_check( &Perm_line_list, &Perm_check, 0, 0 );
-	if( permission == P_REJECT ){
-		DEBUG1("Service_connection: talk socket '%d' no connect perms", talk );
-		Write_fd_str( talk, _("\001no connect permissions\n") );
-		cleanup(0);
-	}
-	Dispatch_input(&talk,input);
-	cleanup(0);
-}
-
-void Dispatch_input(int *talk, char *input )
-{
-	switch( input[0] ){
-		default:
-			FATAL(LOG_INFO)
-				_("Dispatch_input: bad request line '%s'"), input );
-			break;
-		case REQ_START:
-			/* simply send a 0 ACK and close connection - NOOP */
-			Write_fd_len( *talk, "", 1 );
-			break;
-		case REQ_RECV:
-			Receive_job( talk, input );
-			break;
-		case REQ_DSHORT:
-		case REQ_DLONG:
-		case REQ_VERBOSE:
-			Job_status( talk, input );
-			break;
-		case REQ_REMOVE:
-			Job_remove( talk, input );
-			break;
-		case REQ_CONTROL:
-			Job_control( talk, input );
-			break;
-		case REQ_BLOCK:
-			Receive_block_job( talk, input );
-			break;
-		case REQ_SECURE:
-			Receive_secure( talk, input );
-			break;
-#if defined(MIT_KERBEROS4)
-		case REQ_K4AUTH:
-			Receive_k4auth( talk, input );
-			break;
-#endif
-	}
-}
-
 /***************************************************************************
  * Reinit()
  * Reinitialize the database/printcap/permissions information
@@ -710,34 +612,40 @@ int Get_lpd_pid(void)
 	return(pid);
 }
 
-void Set_lpd_pid(void)
+void Set_lpd_pid(int lockfd)
+{
+	/* we write our PID */
+	if( ftruncate( lockfd, 0 ) ){
+		LOGERR_DIE(LOG_ERR) _("lpd: Cannot truncate lock file") );
+	}
+	Server_pid = getpid();
+	DEBUG1( "lpd: writing lockfile fd %d with pid '%d'",lockfd,Server_pid );
+	Write_pid( lockfd, Server_pid, (char *)0 );
+}
+
+int Lock_lpd_pid(void)
 {
 	int lockfd;
 	char *path;
 	struct stat statb;
+	int euid = geteuid();
 
 	path = safestrdup3( Lockfile_DYN,".", Lpd_port_DYN, __FILE__, __LINE__ );
-	lockfd = Checkwrite( path, &statb, O_WRONLY|O_TRUNC, 1, 0 );
+	To_euid_root();
+	lockfd = Checkwrite( path, &statb, O_RDWR, 1, 0 );
+	if( lockfd < 0 ){
+		LOGERR_DIE(LOG_ERR) _("lpd: Cannot open lock file '%s'"), path );
+	}
+#if !defined(__CYGWIN__)
+	fchown( lockfd, DaemonUID, DaemonGID );
 	fchmod( lockfd, (statb.st_mode & ~0777) | 0644 );
-	if( lockfd < 0 ){
-		To_root();
-		lockfd = Checkwrite( path, &statb, O_WRONLY|O_TRUNC, 1, 0 );
-		if( lockfd > 0 ){
-			fchown( lockfd, DaemonUID, DaemonGID );
-			fchmod( lockfd, (statb.st_mode & ~0777) | 0644 );
-		}
-		To_daemon();
+#endif
+	To_euid(euid);
+	if( Do_lock( lockfd, 0 ) < 0 ){
+		close( lockfd );
+		lockfd = -1;
 	}
-	if( lockfd < 0 ){
-		LOGERR_DIE(LOG_ERR) _("lpd: Cannot open '%s'"), path );
-	} else {
-		/* we write our PID */
-		Server_pid = getpid();
-		DEBUG1( "lpd: writing lockfile '%s' fd %d with pid '%d'",path,lockfd,Server_pid );
-		Write_pid( lockfd, Server_pid, (char *)0 );
-	}
-	if(path) free(path); path = 0;
-	close( lockfd );
+	return(lockfd);
 }
 
 int Read_server_status( int fd )
@@ -780,7 +688,7 @@ int Read_server_status( int fd )
 		}
 		buffer[status] = 0;
 		/* we split up read line and record information */
-		Split(&l,buffer,Whitespace,0,0,0,0,0);
+		Split(&l,buffer,Whitespace,0,0,0,0,0,0);
 		if(DEBUGL1)Dump_line_list("Read_server_status - input", &l );
 		for( count = 0; count < l.count; ++count ){ 
 			name = l.list[count];
@@ -805,15 +713,15 @@ int Read_server_status( int fd )
  ***************************************************************************/
 
  static char *msg[] = {
-	N_("usage: %s [-FV] [-D dbg] [-L log]\n"),
+	N_("usage: %s [-FV][-D dbg][-L log][-P path][-p port][-R remote LPD TCP/IP destination port]\n"),
 	N_(" Options\n"),
 	N_(" -D dbg      - set debug level and flags\n"),
-	N_("                 Example: -D10,remote=5\n"),
-	N_("                 set debug level to 10, remote flag = 5\n"),
 	N_(" -F          - run in foreground, log to STDERR\n"),
-	N_("               Example: -D10,remote=5\n"),
 	N_(" -L logfile  - append log information to logfile\n"),
 	N_(" -V          - show version info\n"),
+	N_(" -p port     - TCP/IP listen port, 'off' disables TCP/IP listening port (lpd_listen_port)\n"),
+	N_(" -P path     - UNIX socket path, 'off' disables UNIX listening socket (unix_socket_path)\n"),
+	N_(" -R port     - remote LPD server port (lpd_port)\n"),
 	0,
 };
 
@@ -828,11 +736,13 @@ void usage(void)
 			FPRINTF( STDERR, "%s", _(s) );
 		}
 	}
+	Parse_debug("=",-1);
+	FPRINTF( STDERR, "%s\n", Version );
 	exit(1);
 }
 
  char LPD_optstr[] 	/* LPD options */
- = "D:FL:VX" ;
+ = "D:FL:VX:p:P:" ;
 
 void Get_parms(int argc, char *argv[] )
 {
@@ -843,8 +753,10 @@ void Get_parms(int argc, char *argv[] )
 		case 'D': Parse_debug(Optarg, 1); break;
 		case 'F': Foreground_LPD = 1; break;
 		case 'L': Logfile_LPD = Optarg; break;
-		case 'V': ++verbose; continue;
-		case 'X': Worker_LPD = 1; break;
+		case 'V': ++verbose; break;
+        case 'X': Worker_LPD = Optarg; break;
+		case 'p': Lpd_listen_port_arg = Optarg; break;
+		case 'P': Lpd_socket_arg = Optarg; break;
 		default: usage(); break;
 		}
 	}
@@ -858,20 +770,6 @@ void Get_parms(int argc, char *argv[] )
 	}
 }
 
-
-/*
- * Calls[] = list of dispatch functions 
- */
-
- struct call_list Calls[] = {
-	{&LOGGER,Logger},		/* used by LPD to do logging for remote sites */
-	{&ALL,Service_all},			/* used by LPD for Start_all() operation */
-	{&SERVER,Service_connection},	/* used by LPD to handle a connection */
-	{&QUEUE,Service_queue},		/* used by LPD to handle a queue */
-	{&PRINTER,Service_worker},	/* used by LPD queue manager to do actual printing */
-	{0,0}
-};
-
 int Start_all( int first_scan )
 {
 	struct line_list args, passfd;
@@ -889,14 +787,14 @@ int Start_all( int first_scan )
 	DEBUG1( "Start_all: fd p(%d,%d)",p[0],p[1]);
 
 	Setup_lpd_call( &passfd, &args );
-	Set_str_value(&args,CALL,ALL);
+	Set_str_value(&args,CALL,"all");
 
 	Check_max(&passfd,2);
 	Set_decimal_value(&args,INPUT,passfd.count);
 	passfd.list[passfd.count++] = Cast_int_to_voidstar(p[1]);
 	Set_decimal_value(&args,FIRST_SCAN,first_scan);
 
-	pid = Make_lpd_call( &passfd, &args );
+	pid = Make_lpd_call( "all", &passfd, &args );
 
 	Free_line_list( &args );
 	passfd.count = 0;
@@ -910,80 +808,6 @@ int Start_all( int first_scan )
 	return(p[0]);
 }
 
-void Service_all( struct line_list *args )
-{
-	int i, reportfd, fd, printable, held, move, printing_enabled,
-		server_pid, change;
-	char buffer[SMALLBUFFER], *pr, *forwarding;
-	struct stat statb;
-	int first_scan;
-	
-	/* we start up servers while we can */
-	Name = "STARTALL";
-	setproctitle( "lpd %s", Name );
-
-	first_scan = Find_flag_value(args,FIRST_SCAN,Value_sep);
-	reportfd = Find_flag_value(args,INPUT,Value_sep);
-	Free_line_list(args);
-
-	if(All_line_list.count == 0 ){
-		Get_all_printcap_entries();
-	}
-	for( i = 0; i < All_line_list.count; ++i ){
-		Set_DYN(&Printer_DYN,0);
-		Set_DYN(&Spool_dir_DYN,0);
-		pr = All_line_list.list[i];
-		DEBUG1("Service_all: checking '%s'", pr );
-		if( Setup_printer( pr, buffer, sizeof(buffer), 0) ) continue;
-		/* now check to see if there is a server and unspooler process active */
-		server_pid = 0;
-		if( (fd = Checkread( Printer_DYN, &statb ) ) > 0 ){
-			server_pid = Read_pid( fd, (char *)0, 0 );
-			close( fd );
-		}
-		DEBUG3("Service_all: printer '%s' checking server pid %d", Printer_DYN, server_pid );
-		if( server_pid > 0 && kill( server_pid, 0 ) == 0 ){
-			DEBUG3("Get_queue_status: server %d active", server_pid );
-			continue;
-		}
-		change = Find_flag_value(&Spool_control,CHANGE,Value_sep);
-		printing_enabled = !(Pr_disabled(&Spool_control) || Pr_aborted(&Spool_control));
-
-		Free_line_list( &Sort_order );
-		if( Scan_queue( &Spool_control, &Sort_order,
-				&printable,&held,&move, 1, first_scan, 0 ) ){
-			Close_gdbm();
-			continue;
-		}
-		forwarding = Find_str_value(&Spool_control,FORWARDING,Value_sep);
-		if( change || move || (printable && (printing_enabled||forwarding)) ){
-			if( Server_queue_name_DYN ){
-				pr = Server_queue_name_DYN;
-			} else {
-				pr = Printer_DYN;;
-			}
-			DEBUG1("Service_all: starting '%s'", pr );
-			SNPRINTF(buffer,sizeof(buffer))"%s\n",pr );
-			if( Write_fd_str(reportfd,buffer) < 0 ) cleanup(0);
-		}
-	}
-	Free_line_list( &Sort_order );
-	Errorcode = 0;
-	cleanup(0);
-}
-
-void Service_queue( struct line_list *args )
-{
-	int subserver;
-
-	Set_DYN(&Printer_DYN, Find_str_value(args, PRINTER,Value_sep) );
-	subserver = Find_flag_value( args, SUBSERVER, Value_sep );
-
-	Free_line_list(args);
-	Do_queue_jobs( Printer_DYN, subserver );
-	cleanup(0);
-}
-
 plp_signal_t sigchld_handler (int signo)
 {
 	signal( SIGCHLD, SIG_DFL );
@@ -1000,10 +824,10 @@ void Setup_waitpid_break (void)
 	(void) plp_signal_break(SIGCHLD, sigchld_handler);
 }
 
-void Fork_error( int fork_failed )
+void Fork_error( int last_fork_pid_value )
 {
-	DEBUG1("Fork_error: %d", fork_failed );
-	if( fork_failed < 0 ){
+	DEBUG1("Fork_error: %d", last_fork_pid_value );
+	if( last_fork_pid_value < 0 ){
 		LOGMSG(LOG_CRIT)"LPD: fork failed! LPD not accepting any requests");
 	}
 }
